from dataclasses import dataclass

@dataclass
class PgAssignmentSubcriteriaGrade:
    name: str = "assignment_sassignment_subcriteria_gradesubcriterias"
    subcriteria_grade_id: str = 'subcriteria_grade_id'
    subcriteria_grade_desc: str = 'subcriteria_grade_desc'
